<?php

namespace Drupal\tellstick\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;

/**
 * Defines the Tellstick sensor entity.
 *
 * @ContentEntityType(
 *   id = "tellstick_sensor",
 *   label = @Translation("Tellstick sensor"),
 *   label_singular = @Translation("Tellstick sensor"),
 *   label_plural = @Translation("Tellstick sensors"),
 *   label_count = @PluralTranslation(
 *     singular = "@count Tellstick sensor",
 *     plural = "@count Tellstick sensors"
 *   ),
 *   handlers = {
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\tellstick\TellstickSensorListBuilder",
 *     "views_data" = "Drupal\tellstick\Entity\TellstickSensorViewsData",
 *     "translation" = "Drupal\Core\Entity\EntityTranslationHandler",
 *     "form" = {
 *       "delete" = "Drupal\tellstick\Form\TellstickSensorDeleteForm"
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\tellstick\TellstickSensorHtmlRouteProvider",
 *     },
 *   },
 *   base_table = "tellstick_sensor",
 *   data_table = "tellstick_sensor_field_data",
 *   translatable = TRUE,
 *   admin_permission = "administer tellstick",
 *   entity_keys = {
 *     "id" = "id",
 *     "uuid" = "uuid",
 *     "langcode" = "langecode",
 *   },
 *   links = {
 *     "canonical" = "/admin/structure/tellstick/{tellstick_sensor}",
 *     "delete-form" = "/admin/structure/tellstick/{tellstick_sensor}/delete",
 *     "collection" = "/admin/structure/tellstick"
 *   }
 * )
 */
class TellstickSensor extends ContentEntityBase implements TellstickSensorInterface {

  /**
   * {@inheritdoc}
   */
  public function getSensorType() {
    return $this->get('sensorType')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function getSensor() {
    return $this->get('sensor')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function getPlace() {
    return $this->get('place')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function getCreated() {
    return \Drupal::service('date.formatter')
      ->format($this->get('created')->value);
  }

  /**
   * {@inheritdoc}
   */
  public function getChanged() {
    return \Drupal::service('date.formatter')
      ->format($this->get('changed')->value);
  }

  /**
   * {@inheritdoc}
   */
  public function getIgnored() {
    return $this->get('ignored')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setIgnored($ignored) {
    return $this->set('ignored', $ignored);
  }

  /**
   * {@inheritdoc}
   */
  public function getWeight() {
    return $this->get('weight')->value;
  }

  /**
   * Set the weight.
   *
   * @param string $weight
   *   The weight to set.
   *
   * @return string $weight
   *   Return the value.
   */
  public function setWeight($weight) {
    return $this->set('weight', $weight);
  }

  /**
   * Get the rain since midnight.
   *
   * @return string
   *   The rain since midnight.
   */
  public function getSinceMidnight() {
    return $this->getData("sinceMidnight");
  }

  /**
   * Get the battery value and normalise it.
   */
  public function getBattery() {
    $tdata = $this->get('data')->get(0)->getValue();
    if (empty($tdata['Battery_max']) ||
      $tdata['Battery'] > $tdata['Battery_max']) {
      $this->setData('Battery_max', $tdata['Battery']);
      $this->save();
      $tdata = $this->get('data')->get(0)->getValue();
    }
    return round($tdata['Battery'] * 100 / $tdata['Battery_max']);
  }

  /**
   * Get data from the generic data storage.
   */
  public function getData($name) {
    $tdata = $this->get('data')->get(0)->getValue();
    return !empty($tdata[$name]) ? $tdata[$name] : NULL;;
  }

  /**
   * Set a value in the generic data storage.
   */
  public function setData($name, $value) {
    $tdata = $this->get('data')->get(0)->getValue();
    $tdata[$name] = (string) $value;
    $this->set('data', serialize($tdata));
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['sensor'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Sensor Name'))
      ->setDescription(t('The sensor name.'))
      ->setDisplayOptions('view', ['weight' => 0]);

    $fields['sensorType'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Sensor Type'))
      ->setDescription(t('The sensor Type.'))
      ->setDisplayOptions('view', ['weight' => 2]);

    $fields['ignored'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Sensor hidden'))
      ->setDescription(t('Is the Sensor visible in the block.'))
      ->setDefaultValue(TRUE)
      ->setDisplayOptions('view', ['weight' => 3])
      ->setSetting('off_label', 'No')
      ->setSetting('on_label', 'Yes');

    $fields['place'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Placement'))
      ->setDescription(t('The; place for the controller.'))
      ->setDisplayOptions('view', ['weight' => 1]);

    $fields['weight'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Weight'))
      ->setDescription(t('The sensor weight.'))
      ->setDisplayOptions('view', ['weight' => 7]);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Sensor created'))
      ->setDescription(t('When was the sensor created.'))
      ->setDisplayOptions('view', ['weight' => 5]);

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Last changed'))
      ->setDescription(t('The last update of the sensor.'))
      ->setDisplayOptions('view', ['weight' => 6]);

    $fields['data'] = BaseFieldDefinition::create('map')
      ->setLabel(t('Serialized data'))
      ->setDescription(t('The Serialized data.'));

    return $fields;
  }

}
