<?php

namespace Drupal\tellstick\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Cache\Cache;
use Drupal\Component\Utility\SortArray;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * {@inheritdoc}
 */
class TellstickBlockSettingForm extends ConfigFormBase {

  // TODO: Remove when included in the TellstickSensorListBuilder class.

  /**
   * The Telstick Live service.
   *
   * @var \Drupal\tellstick\TelltickLiveService
   */
  protected $tellstickLiveService;

  /**
   * The Tellstick service.
   *
   * @var \Drupal\tellstick\TelltickService
   */
  protected $tellstickService;

  /**
   * Constructs a \Drupal\system\ConfigFormBase object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    parent::__construct($config_factory);
    $this->tellstickLiveService =
      \Drupal::service('tellstick.live');
    $this->tellstickService =
      \Drupal::service('tellstick');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['tellstick.block_settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'tellstick_block_setting_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('tellstick.block_settings');
    $credentials = $this->config('tellstick.settings');
    $tellstickService = \Drupal::service('tellstick');
    $tellstickLiveService = \Drupal::service('tellstick.live');
    $sxe = $tellstickLiveService->getTellstickLiveSensors();

    $form['#tree'] = TRUE;
    $header = [
      'name' => $this->t('Sensor'),
      'visible' => $this->t('Visibility in block'),
      'weight' => $this->t('Weight'),
      'configure' => $this->t('Operations'),
    ];
    $form['sensors'] = [];

    foreach ($sxe->children() as $order => $sensor) {
      $attributes = $sensor->attributes();
      $id = (string) $attributes['id'];
      if (empty($attributes['name'])) {
        continue;
      }
      $active = !$tellstickService->getTellstickSensorById($id)->getIgnored();
      $weight = $tellstickService->getTellstickSensorById($id)->getWeight();

      $form['sensors'][$id]['#attributes']['class'][] = 'draggable';
      $form['sensors'][$id]['name'] = [
        '#type' => 'item',
        '#markup' => $attributes['name']->__toString(),
      ];

      $form['sensors'][$id]['visible'] = [
        '#type' => 'item',
        '#markup' => $active ? $this->t('Visible') : $this->t('Hidden'),
      ];

      $form['sensors'][$id]['weight'] = [
        '#type' => 'weight',
        '#default_value' => $weight,
        '#title' => $this->t('Weight'),
        '#title_display' => 'invisible',
        '#attributes' => [
          'class' => [
            'table-sort-weight',
          ],
        ],
      ];

      $url1 = Url::fromRoute('entity.tellstick_sensor.canonical',
        ['tellstick_sensor' => $id]);
      $links['view'] = [
        'title' => $this->t('View'),
        'url' => $url1,
      ];
      $url2 = Url::fromRoute('entity.tellstick_sensor.toggle',
        ['tellstick_sensor' => $id]);
      $links['toggle'] = [
        'title' => $active ? $this->t('Hide') : $this->t('Unhide'),
        'url' => $url2,
      ];
      $form['sensors'][$id]['configure'] = [
        '#type' => 'operations',
        '#links' => $links,
      ];
    }

    uasort($form['sensors'],
      [SortArray::class, 'sortByWeightElement']);

    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save changes'),
      '#button_type' => 'primary',
    ];

    $form['sensors'] += [
      '#type' => 'table',
      '#header' => $header,
      '#empty' => $this->t('No sensors found'),
      '#tabledrag' => [
        [
          'action' => 'order',
          'relationship' => 'sibling',
          'group' => 'table-sort-weight',
        ],
      ],
      '#cache' => [
        'max-age' => 300,
      ],
    ];

    return $form;

  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    foreach ($form_state->getValues()['sensors'] as $id => $data) {
      if (is_array($data) && isset($data['weight'])) {
        $sensor = $this->tellstickService->getTellstickSensorById($id);
        // Only update if this is a form element with weight.
        $sensor->setWeight($data['weight']);
        $sensor->save();
      }
    }
    // Reset cache.
    Cache::invalidateTags(['entity_view:block:tellsticksensors']);
    // Set message.
    $this
      ->messenger()
      ->addStatus($this
        ->t('The sensor ordering has been saved.')
    );
  }

}
