<?php

namespace Drupal\tellstick\Plugin\views\field;

use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\tellstick\TellstickSensorHelper;

/**
 * A field that displays the sensor raw data in different ways.
 *
 * @ingroup views_field_handlers
 *
 * @ViewsField("tdata")
 */
class TellstickHandlerFieldData extends FieldPluginBase {

  const IGNORE_VALUES =
      'id name lastUpdated ignored client clientName online editable keepHistory protocol model sensorId miscValues';

  /**
   * The Config Factory.
   *
   * @var ConfigFactory
   *   The config factory.
   */
  protected $configFatory;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration,
                                    $plugin_id,
                                    $plugin_definition) {
    parent::__construct($configuration,
                        $plugin_id,
                        $plugin_definition);
    $this->configFactory = \Drupal::service('config.factory');
  }

  /**
   * {@inheritdoc}
   */
  public function defineOptions() {
    $options = parent::defineOptions();

    $options['dataType'] = ['default' => 'formatted'];
    $options['raw'] = ['default' => 'Undefined'];

    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);
    $form['dataType'] = [
      '#type' => 'select',
      '#title' => $this->t('Output format'),
      '#options' => [
        'formatted' => $this->t('Formatted'),
        'serialized' => $this->t('Serialized'),
        'unserialized' => $this->t('Unserialized'),
        'raw' => $this->t('Raw'),
      ],
      '#description' => $this->t('Formatted gives a value directly for outputting, raw gives a value that can be used for creating diagrams'),
      '#default_value' => $this->options['dataType'],
    ];
    $form['raw'] = [
      '#type' => 'select',
      '#title' => $this->t('Raw data'),
      '#options' => $this->getRawOptionsAvailable(),
      '#default_value' => $this->options['raw'],
      '#description' => $this->t('Make sure to select a valid item for the sensor in question'),
      '#dependency' => [
        'edit-options-type' => ['raw'],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function render($items) {
    $item = $this->getValue($items);

    switch ($this->options['dataType']) {
      case 'serialized':
        break;

      case 'unserialized':
        $item = print_r(unserialize($item), TRUE);
        break;

      case 'raw':
        $item = $this->getDataRaw($item);
        break;

      case 'formatted':
      default:
        $item = $this->getDataValue($item);
    }
    return $this->sanitizeValue($item);
  }

  /**
   * {@inheritdoc}
   */
  private function getDataValue($value) {
    $unserial = unserialize($value);
    $data = '';
    $dew_calc = [];
    $compare_string = self::IGNORE_VALUES;
    if ($this->configFactory->get('tellstick')->get('battery_status') == 0) {
      $compare_string = $compare_string . ' battery';
    }
    foreach ($unserial as $key => $temp_value) {
      if (strpos($compare_string, $key) === FALSE) {
        $dew_calc[$key] = $temp_value;
        $data .= TellstickSensorHelper::sensorPost($key,
          $temp_value);
        if (!empty($data)) {
          $data .= '<br />';
        }
      }
    }
    if (!empty($dew_calc['temp']) & !empty($dew_calc['humidity'])) {
      $data .= TellstickSensorHelper::sensorPost('dewpt',
        round(TellstickSensorHelper::dewPoint($dew_calc['temp'],
        $dew_calc['humidity']), 1));
      $data .= '<br />';
    }
    return $data;
  }

  /**
   * {@inheritdoc}
   */
  private function getDataRaw($value) {
    $unserial = unserialize($value);
    $data = [];
    $dew_calc = [];
    $compare_string = self::IGNORE_VALUES;
    if ($this->configFactory->get('tellstick.sensor_settings')
      ->get('battery_status') == 0) {
      $compare_string = $compare_string . ' battery';
    }
    foreach ($unserial as $key => $temp_value) {
      if (strpos($compare_string, $key) === FALSE) {
        $dew_calc[$key] = $temp_value;
        $data[$key] = (double) $temp_value;
      }
    }
    if (!empty($dew_calc['temp']) & !empty($dew_calc['humidity'])) {
      $data['dewpt'] = round(TellstickSensorHelper::dewPoint($dew_calc['temp'],
        $dew_calc['humidity']), 1);
    }
    return empty($data[$this->options['raw']]) ? FALSE : $data[$this->options['raw']];
  }

  /**
   * {@inheritdoc}
   */
  private function getRawOptionsAvailable() {
    $options = [
      /* Would be better to grab the info from the selected sensor
         but until then we do it predefined */
      'temp' => t('Temperature'),
      'humidity' => t('Humidity'),
      'dewpt' => t('Dew point'),
      'rrate' => t('Rain / hour'),
      'rday' => t('Rain / day'),
      'rtot' => t('Total rain'),
      'wavg' => t('Wind average'),
      'wgust' => t('Wind gust'),
      'wdir' => t('Wind direction'),
      'battery' => t('Battery'),
    ];
    return $options;
  }

}
