<?php

namespace Drupal\tellstick;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\tellstick\Entity\TellstickSensor;

/**
 * Class Tellstick service.
 *
 *  Handles search requests for Tellstick Sensors.
 */
class TellstickService implements TellstickServiceInterface {

  /**
   * The entity query manager injected into the service.
   *
   * @var \Drupal\Core\Entity\Query\QueryFactory
   */
  private $entityQueryManager;

  /**
   * The current user injected into the service.
   *
   * @var Drupal\Core\Session\AccountInterface
   */
  private $currentUser;

  /**
   * The type Entity Type Manager injected into the service.
   *
   * @var Drupal\Core\Entity\EntityTypeManagerInterface
   */
  private $entityTypeManager;

  /**
   * The Tellstick Live Sensor Connector.
   *
   * @var Drupal\tellstick\TellstickLiveService
   */
  private $tellstickLiveService;

  /**
   * Constructor.
   *
   * @param Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity manager.
   * @param Drupal\tellstick\TellstickLiveService $tellstick_live_service
   *   The entity manager.
   */
  public function __construct(AccountInterface $current_user,
                              EntityTypeManagerInterface $entity_type_manager,
                              TellstickLiveService $tellstick_live_service) {
    $this->currentUser = $current_user;
    $this->entityTypeManager = $entity_type_manager;
    $this->tellstickLiveService = $tellstick_live_service;
  }

  /**
   * Get all Tellstick Sensors.
   *
   * @return Drupal\tellstick\Entity\TellstickSensor[]
   *   An array of found Flag For Lists.
   */
  public function getAllTellstickSensors() {
    $query = $this->entityTypeManager
      ->getStorage('tellstick_sensor')
      ->getQuery();
    $ids = $query->execute();

    return $ids;
  }

  /**
   * Get a Tellstick Sensor by Id.
   *
   * @param string $sensor_id
   *   The Sensor Id of the Tellstick Sensor to load.
   *
   * @return Drupal\tellstick\Entity\TellstickSensor
   *   The loaded Telsstick Sensor.
   */
  public function getTellstickSensorById($sensor_id) {
    $nowTime = Drupal::time()->getRequestTime();
    // First try with the Sensor Entity.
    $sensor = $this->entityTypeManager
      ->getStorage('tellstick_sensor')
      ->load($sensor_id);
    // If this is the first time force an update.
    // If it is more than 1 h also force an update.
    $update = empty($sensor) ? TRUE :
      $sensor->get('changed')->value < \Drupal::time()->getRequestTime() - 1 * 60 * 60;
    if ($update) {
      // Try first loading from the Telldus Live instead.
      $live_sensor = $this->tellstickLiveService
        ->getTellstickLiveSensorInfo($sensor_id);
      if (!empty($live_sensor)) {
        // Save new data into TellstickSensor entity.
        $name = $live_sensor['name'];
        $model = $live_sensor['model'];
        $clientName = $live_sensor['clientName'];
        $ignored = $live_sensor['ignored'];
        $changed = $live_sensor['lastUpdated'];

        $battery = $live_sensor['battery'];

        if (empty($sensor)) {
          // This is the first time for this sensor.
          //
          // Remember the highest battery value and use it
          // as scaling factor.
          // (Sensor doesn't exist so there is no max value.)
          $data = [
            'Battery' => (string) $battery,
            'Battery_max' => (string) $battery,
          ];
          $tellstick_sensor = TellstickSensor::create([
            'id' => $sensor_id,
            'sensor' => (string) $name,
            'sensorType' => (string) $model,
            'place' => (string) $clientName,
            // Use the Tellsticke Live! setting as default value.
            'ignored' => (string) $ignored,
            // Use the last update from Tellstick Live!.
            'changed' => (string) $changed,
            'created' => \Drupal::time()->getRequestTime(),
            'weight' => 0,
            'data' => serialize($data),
          ]);
          $tellstick_sensor->save();
          // Now when it is loaded and stored, recursivly call
          // ourself again.
          $sensor = $this->getTellstickSensorById($sensor_id);
        }
        else {
          // The sensor already exists, so update instead.
          $sensor->set('sensor', (string) $name);
          $sensor->set('sensorType', (string) $model);
          $sensor->set('place', (string) $clientName);
          $sensor->setData('Battery', $battery);
          // Only update the changed time but not the created.
          $sensor->set('changed', (string) $changed);
          // $sensor->set('created', (string) $changed);
          $sensor->save();
        }
      }
    }
    return $sensor;
  }

  /**
   * Loads Tellstick Sensors by ids.
   *
   * @param int[] $ids
   *   The Ids of the Tellstick Sensors to load.
   *
   * @return Drupal\tellstick\Entity\TellstickSensor
   *   The loaded Tellstick Sensors.
   */
  protected function getTellstickSensorByIds(array $ids) {
    return $this->entityTypeManager->getStorage('tellstick_sensor')->loadMultiple($ids);
  }

  /**
   * Get a Tellstick Data Item.
   *
   * @param int $id
   *   The Tellstick Data Item ids to load.
   *
   * @return Drupal\tellstick\Entity\TellstickData[]
   *   A Tellstick Data Item.
   */
  public function getTellstickDataById($id) {
    return $this->entityTypeManager->getStorage('tellstick_data')->load($id);
  }

  /**
   * Get Tellstick Data Items.
   *
   * @param int[] $ids
   *   The Tellstick Data Item ids to load.
   *
   * @return Drupal\tellstick\Entity\TellstickData[]
   *   An array of Tellstick Data Items.
   */
  public function getTellstickDataByIds(array $ids) {
    return $this->entityTypeManager->getStorage('tellstick_data')->loadMultiple($ids);
  }

}
